@@ -5,6 +5,7 @@
  */
 
 typedef Display* DisplayOrNull; // Used by typemap for stricter conversion
+typedef Visual* VisualOrNull;
 typedef int ScreenNumber; // used by typemap to coerce X11::Xlib::Screen
 
 // Methods to create/alter the magic Display* attached to X11::Xlib objects
@@ -15,6 +16,11 @@ extern SV * PerlXlib_obj_for_display(Display *dpy, int create);
 
 extern XID PerlXlib_sv_to_xid(SV *sv);
 
+extern int PerlXlib_keysym_to_codepoint(KeySym keysym);
+extern KeySym PerlXlib_codepoint_to_keysym(int codepoint);
+extern SV * PerlXlib_keysym_to_sv(KeySym keysym, int symbolic);
+extern KeySym PerlXlib_sv_to_keysym(SV *sv);
+
 typedef void PerlXlib_struct_pack_fn(void*, HV*, Bool consume);
 
 extern void* PerlXlib_get_struct_ptr(SV *sv, int lvalue, const char* pkg, int struct_size, PerlXlib_struct_pack_fn *packer);
