@@ -8,7 +8,7 @@ use base qw(Exporter DynaLoader);
 use Carp;
 use Try::Tiny;
 
-our $VERSION = '0.09_05';
+our $VERSION = '0.09_06';
 
 sub dl_load_flags { 1 } # Make PerlXLib.c functions available to other XS modules
 
@@ -29,9 +29,6 @@ my %_constants= (
     KeymapNotify LeaveNotify MapNotify MappingNotify MotionNotify NoExpose
     PropertyNotify ReparentNotify ResizeRequest SelectionClear SelectionNotify
     SelectionRequest UnmapNotify VisibilityNotify )],
-  const_event_input_state => [qw( Button1Mask Button2Mask Button3Mask
-    Button4Mask Button5Mask ControlMask LockMask Mod1Mask Mod2Mask Mod3Mask
-    Mod4Mask Mod5Mask ShiftMask )],
   const_event_mask => [qw( Button1MotionMask Button2MotionMask
     Button3MotionMask Button4MotionMask Button5MotionMask ButtonMotionMask
     ButtonPressMask ButtonReleaseMask ColormapChangeMask EnterWindowMask
@@ -40,6 +37,11 @@ my %_constants= (
     PointerMotionMask PropertyChangeMask ResizeRedirectMask
     StructureNotifyMask SubstructureNotifyMask SubstructureRedirectMask
     VisibilityChangeMask )],
+  const_input => [qw( AnyKey AnyModifier AsyncBoth AsyncKeyboard AsyncPointer
+    Button1Mask Button2Mask Button3Mask Button4Mask Button5Mask ControlMask
+    GrabModeAsync GrabModeSync LockMask Mod1Mask Mod2Mask Mod3Mask Mod4Mask
+    Mod5Mask NoSymbol ReplayKeyboard ReplayPointer ShiftMask SyncBoth
+    SyncKeyboard SyncPointer XK_VoidSymbol )],
   const_sizehint => [qw( PAspect PBaseSize PMaxSize PMinSize PPosition
     PResizeInc PSize PWinGravity USPosition USSize )],
   const_visual => [qw( VisualAllMask VisualBitsPerRGBMask VisualBlueMaskMask
@@ -59,13 +61,15 @@ my %_functions= (
   fn_event => [qw( XCheckMaskEvent XCheckTypedEvent XCheckTypedWindowEvent
     XCheckWindowEvent XFlush XNextEvent XPutBackEvent XSelectInput XSendEvent
     XSync )],
-  fn_input => [qw(  )],
+  fn_input => [qw( XAllowEvents XBell XGrabButton XGrabKey XGrabKeyboard
+    XGrabPointer XQueryKeymap XUngrabButton XUngrabKey XUngrabKeyboard
+    XUngrabPointer keyboard_leds )],
   fn_keymap => [qw( XDisplayKeycodes XGetKeyboardMapping XGetModifierMapping
     XKeysymToKeycode XLookupString XRefreshKeyboardMapping XSetModifierMapping
-    )],
+    load_keymap save_keymap )],
   fn_keysym => [qw( IsFunctionKey IsKeypadKey IsMiscFunctionKey IsModifierKey
     IsPFKey IsPrivateKeypadKey XConvertCase XKeysymToString XStringToKeysym
-    )],
+    char_to_keysym codepoint_to_keysym keysym_to_char keysym_to_codepoint )],
   fn_pix => [qw( XCreateBitmapFromData XCreatePixmap
     XCreatePixmapFromBitmapData XFreePixmap )],
   fn_screen => [qw( DefaultColormap DefaultDepth DefaultGC DefaultScreen
@@ -76,8 +80,8 @@ my %_functions= (
   fn_win => [qw( XCreateSimpleWindow XCreateWindow XDestroyWindow XGetGeometry
     XGetWMNormalHints XGetWMSizeHints XMapWindow XSetWMNormalHints
     XSetWMSizeHints XUnmapWindow )],
-  fn_xtest => [qw( XBell XQueryKeymap XTestFakeButtonEvent XTestFakeKeyEvent
-    XTestFakeMotionEvent )],
+  fn_xtest => [qw( XTestFakeButtonEvent XTestFakeKeyEvent XTestFakeMotionEvent
+    )],
 # END GENERATED XS FUNCTION LIST
 );
 our @EXPORT_OK= map { @$_ } values %_constants, values %_functions;
@@ -194,8 +198,17 @@ X11::Xlib - Low-level access to the X11 library
   
   use X11::Xlib;
   my $display= X11::Xlib->new($conn_string);  # shortcut for X11::Xlib::Display->new
-  my $window= $display->new_window({ x => 0, y => 0, width => 50, height => 50);
-  $window->show();
+  $display->fake_motion(undef, 50, 50)
+  $display->flush;
+
+  # Remap Caps_Lock to a Smiley face
+  
+  use X11::Xlib;
+  my $display= X11::Xlib->new;
+  my $caps_code= $display->keymap->find_keycode('Caps_lock') // 0x42;
+  $display->keymap->modmap_del_codes(lock => $caps_code);
+  $display->keymap->keymap->[$caps_code]= [("U263A") x 4];
+  $display->keymap->save;
 
 =head1 DESCRIPTION
 
@@ -213,8 +226,6 @@ on that, you get a more friendly wrapper around Xlib that helps you manage
 resource lifespan, wraps XIDs with perl objects, and does some sanity checking
 on the state of the library when you call methods.
 
-=cut
-
 =head1 ATTRIBUTES
 
 The X11::Xlib connection is a hashref with a few attributes and methods
@@ -222,7 +233,7 @@ independent of Xlib.
 
 =head2 autoclose
 
-Boolean flag that determines whether the destructor will call XCloseDisplay.
+Boolean flag that determines whether the destructor will call L</XCloseDisplay>.
 Defaults to true for connections returned by L</XOpenDisplay>.
 
 =head1 FUNCTIONS
@@ -252,7 +263,7 @@ would have to perform all cleanup with your stack tied up through Xlib, but
 this library cheats by using croak (C<longjmp>) to escape the callback and let
 you wrap up your script in a normal manner.  B<However>, after a fatal
 error Xlib's internal state could be damaged, so it is unsafe to make any more
-Xlib calls.  The library tries to help assert this by invalidating all the
+Xlib calls.  This library tries to help enforce that by invalidating all the
 connection objects.
 
 If you really need your program to keep running your best bet is to state-dump
@@ -595,7 +606,7 @@ C<< $visual_info->depth >> as the C<$depth> parameter, and create a matching
 L</Colormap> which you pass via the C<\%attrs> parameter.
 
 Since this function didn't have nearly enough parameters for the imaginations
-of the Xlib creators, they added the full L</XSetWindowAttributes> structure
+of the Xlib creators, they added the full L<X11::Xlib::XSetWindowAttributes> structure
 as a final argument.  But to save you the trouble of setting all I<those>
 fields, they added an C<$attr_mask> to indicate which fields you are using.
 
@@ -705,6 +716,96 @@ and C<$pressed> indicates if the key was pressed or released.
 The optional C<$EventSendDelay> parameter specifies the number of milliseconds to wait
 before sending the event. The default is 10 milliseconds.
 
+=head2 KEYSYM FUNCTIONS
+
+These utility functions help identify and convert KeySym values, and do not
+depend on a connection to an X server.
+
+=head3 XKeysymToString
+
+  XKeysymToString($keysym)
+
+Return the human-readable string for character number C<$keysym>.
+
+C<XKeysymToString> is the exact reverse of C<XStringToKeysym>.
+
+=head3 XStringToKeysym
+
+  XStringToKeysym($string)
+
+Return the keysym number for the human-readable character C<$string>.
+
+C<XStringToKeysym> is the exact reverse of C<XKeysymToString>.
+
+=head3 codepoint_to_keysym
+
+  my $keysym= codepoint_to_keysym(ord($char));
+
+Convert a Unicode codepoint to a keysym value.  This isn't a true Xlib
+function, but fills a gap in the API.  Every normal unicode codepoint has a
+keysym value, but if you pass an invalid codepoint you will get C<undef>.
+
+=head3 keysym_to_codepoint
+
+  my $cp= keysym_to_codepoint($keysym);
+  my $char= defined $cp? chr($cp) : undef;
+
+Convert a KeySym to a unicode codepoint.  Many KeySyms (like F1, Control, etc)
+do not have any character associated with them, and will return C<undef>.
+
+=head3 char_to_keysym
+
+Like L</codepoint_to_keysym> example above, but takes a string from which it
+calls L<ord()> on the first character.  Returns undef if the string doesn't
+have a first character.
+
+=head3 keysym_to_char
+
+Like L</keysym_to_codepoint> example above, but returns a string if there is
+a valid codepoint, and C<undef> otherwise.
+
+=head3 XConvertCase
+
+  XConvertCase($keysym, $lowercase_out, $uppercase_out);
+
+Return the lowercase and uppercase KeySyms values for C<$keysym>.
+
+=head3 IsFunctionKey
+
+  IsFunctionKey($keysym)
+
+Return true if C<$keysym> is a function key (F1 .. F35)
+
+=head3 IsKeypadKey
+
+  IsKeypadKey($keysym)
+
+Return true if C<$keysym> is on numeric keypad.
+
+=head3 IsMiscFunctionKey
+
+  IsMiscFunctionKey($keysym)
+
+Return true is key if... honestly don't know :\
+
+=head3 IsModifierKey
+
+  IsModifierKey($keysym)
+
+Return true if C<$keysym> is a modifier key (Shift, Alt).
+
+=head3 IsPFKey
+
+  IsPFKey($keysym)
+
+Xlib docs are fun.  No mention of what "PF" might be.
+
+=head3 IsPrivateKeypadKey
+
+  IsPrivateKeypadKey($keysym)
+
+True for vendor-private key codes.
+
 =head2 KEYCODE FUNCTIONS
 
 =head3 XQueryKeymap
@@ -713,6 +814,58 @@ before sending the event. The default is 10 milliseconds.
 
 Return a list of the key codes currently pressed on the keyboard.
 
+=head3 XGrabKeyboard
+
+  $bool= XGrabKeyboard($display, $window, $owner_events, $pointer_mode, $keyboard_mode, $timestamp)
+
+=head3 XUngrabKeyboard
+
+  XUngrabKeyboard($display, $timestamp)
+
+=head3 XGrabKey
+
+  XGrabKey($display, $keycode, $modifiers, $window, $owner_events, $pointer_mode, $keyboard_mode)
+
+Register a window to receive any matching key events, optionally hiding them
+from the normal target window.
+
+C<$keycode> is the keyboard scan code to watch for, or L<AnyKey>.
+C<$modifiers> is a bit mask combined from C<ControlMask>, C<LockMask>,
+C<Mod1Mask>, C<Mod2Mask>, C<Mod3Mask>, C<Mod4Mask>, C<Mod5Mask>, C<ShiftMask>,
+or the special mask C<AnyModifier> which means any I<or none> of the modifiers.
+C<$window> is the XID or L<X11::Xlib::Window> to direct events toward.
+C<$owner_events> is a boolean of whether to also let the normal target of the
+key event receive them.  C<$pointer_mode> and C<$keyboard_mode> are either
+C<GrabModeSync> or C<GrabModeAsync>.
+
+=head3 XUngrabKey
+
+  XUngrabKey($display, $keycode, $modifiers, $window)
+
+Cancel a grab registered by L</XGrabKey>.
+
+=head3 XGrabPointer
+
+  $bool= XGrabPointer($display, $window, $owner_events, $event_mask,
+    $pointer_mode, keyboard_mode, confine_to, cursor, timestamp)
+
+=head3 XUngrabPointer
+
+  XUngrabPointer(dpy, timestamp)
+
+=head3 XGrabButton
+
+  XGrabButton($display, $button, $modifiers, $window, $owner_events,
+    $event_mask, $pointer_mode, $keyboard_mode, $confine_to, $cursor)
+
+=head3 XUngrabButton
+
+  XUngrabButton($display, $button, $modifiers, $window)
+
+=head3 XAllowEvents
+
+  XAllowEvents($display, $event_mode, $timestamp)
+
 =head3 XGetKeyboardMapping
 
   XGetKeyboardMapping($display, $keycode, $count)
@@ -725,7 +878,63 @@ Lock, Mode).  The X11 server may return a variable number of codes per key,
 which you can determine by dividing the total number of values returned by this
 function by the C<$count>.
 
-For an easier interface, see L<X11::Xlib::Keymap>.
+For a more perl-friendly interface, see L</load_keymap>.  For object-oriented
+access, see L<X11::Xlib::Keymap>.
+
+=head3 XSetKeyboardMapping
+
+  XChangeKeyboardMapping($display, $first_keycode, $keysym_per_keycode, \@keysyms, $num_codes);
+  
+  # Best explained with an example...
+  my @keycodes= (
+    ord('a'), ord('A'), ord('a'), ord('A'),  # want to assign these to KeyCode 38
+    ord('s'), ord('S'), ord('s'), ord('S'),  # and these to KeyCode 39
+  );
+  XChangeKeyboardMapping($display, 38, 4, \@keycodes, scalar @keycodes);
+
+Update some/all of the KeySyms attached to one or more KeyCodes.
+In the example above, only the first 4 KeySyms of each KeyCode will be changed.
+Specify a larger number of C<$keysym_per_keycode> to overwrite more of them.
+
+For a more perl-friendly interface, see L</save_keymap>.  For object-oriented
+access, see L<X11::Xlib::Keymap>.
+
+=head3 load_keymap
+
+  my $keymap= load_keymap($display, $symbolic, $min_key, $max_key);
+  my $keymap= load_keymap($display, $symbolic); # all keys
+  my $keymap= load_keymap($display); # all keys, symbolic=2
+
+This is a wrapper around L</XGetKeyboardMapping> which returns an arrayref
+of arrayrefs, and also translates KeySym values into KeySym names or unicode
+characters.  If C<$symbolic> is 0, the elements of the arrays are numbers.
+If C<$symbolic> is 1, the elements are the KeySym name (or integers, if a name
+is not available).
+If C<$symbolic> is 2, the elements are characters for every KeySym that can be
+un-ambiguously represented by a character, else KeySym names, else integers.
+
+The minimum KeyCode of an X server is never below 8.  If you omit C<$min_key>
+it defaults to 0, and so the returned array will always have at least 8 undef
+values at the start.  This minor waste allows you to index into the array
+directly with a KeyCode.
+
+=head3 save_keymap
+
+  save_keymap($display, \@keymap, $min_key, $max_key);
+  save_keymap($display, \@keymap); # to update all keycodes
+
+This is a wrapper around L</XChangeKeyboardMapping> that accepts the same
+array-of-arrays returned by L</load_keymap>.  The first element of the array
+is assumed to be C<$min_key> B<unless the array is longer than C<$max_key> >
+in which case the array is assumed to start at 0 and you are requesting that
+only elements C<($min_key .. $max_key)> be sent to the X server.
+
+Each element of the inner array can be an integer KeySym, or a KeySym name
+recognized by L</XStringToKeysym>, or a single unicode character.
+If the KeySym is an integer, it must be at least two integer digits, which
+all real KeySyms should be (other that NoSymbol which has the value 0, and
+should be represented by C<undef>) to avoid ambiguity with the characters of
+the number keys.
 
 =head3 XGetModifierMapping
 
@@ -772,64 +981,6 @@ overwritten with the numeric value of the KeySym.
 
 Return the key code corresponding to the character number C<$keysym>.
 
-=head3 XKeysymToString
-
-  XKeysymToString($keysym)
-
-Return the human-readable string for character number C<$keysym>.
-
-C<XKeysymToString> is the exact reverse of C<XStringToKeysym>.
-
-=head3 XStringToKeysym
-
-  XStringToKeysym($string)
-
-Return the keysym number for the human-readable character C<$string>.
-
-C<XStringToKeysym> is the exact reverse of C<XKeysymToString>.
-
-=head3 XConvertCase
-
-  XConvertCase($keysym, $lowercase_out, $uppercase_out);
-
-Return the lowercase and uppercase KeySyms values for C<$keysym>.
-
-=head3 IsFunctionKey
-
-  IsFunctionKey($keysym)
-
-Return true if C<$keysym> is a function key (F1 .. F35)
-
-=head3 IsKeypadKey
-
-  IsKeypadKey($keysym)
-
-Return true if C<$keysym> is on numeric keypad.
-
-=head3 IsMiscFunctionKey
-
-  IsMiscFunctionKey($keysym)
-
-Return true is key if... honestly don't know :\
-
-=head3 IsModifierKey
-
-  IsModifierKey($keysym)
-
-Return true if C<$keysym> is a modifier key (Shift, Alt).
-
-=head3 IsPFKey
-
-  IsPFKey($keysym)
-
-Xlib docs are fun.  No mention of what "PF" might be.
-
-=head3 IsPrivateKeypadKey
-
-  IsPrivateKeypadKey($keysym)
-
-True for vendor-private key codes.
-
 =head3 XBell
 
   XBell($display, $percent)
@@ -950,7 +1101,7 @@ Pure-perl implementation of the X11 protocol.
 
 =head1 TODO
 
-This module still only covers a small fraction of the Xlib API.
+This module still only covers a fraction of the Xlib API.
 Patches are welcome :)
 
 =head1 AUTHOR
