@@ -10,7 +10,9 @@ require X11::Xlib::Colormap;
 require X11::Xlib::Window;
 require X11::Xlib::Pixmap;
 
-# ABSTRACT - Object-Oriented behavior for X11::Xlib
+=head1 NAME
+
+X11::Xlib::Display - Object-Oriented behavior for X11::Xlib
 
 =head1 DESCRIPTION
 
@@ -558,6 +560,10 @@ sub new_window {
     $y ||= 0;
     $w ||= $args{min_width} || 0;
     $h ||= $args{min_height} || 0;
+    $border ||= 0;
+    $depth= X11::Xlib::CopyFromParent unless defined $depth;
+    $class= X11::Xlib::CopyFromParent unless defined $class;
+    $visual= X11::Xlib::CopyFromParent unless defined $visual;
 
     # Now extract fields specific to XSizeHints
     my %sizehints;
@@ -570,11 +576,8 @@ sub new_window {
 
     my $wnd= $self->XCreateWindow(
         $args{parent} || $self->root_window,
-        $x, $y, $w, $h,
-        $border || 0,
-        $depth || $self->depth,
-        (defined $class? $class : X11::Xlib::CopyFromParent),
-        $visual || $self->visual,
+        $x, $y, $w, $h, $border,
+        $depth, $class, $visual,
         $attrflags, \%attrs
     );
 
@@ -591,6 +594,16 @@ sub RootWindow {
     $_[0]->get_cached_window( &X11::Xlib::RootWindow );
 }
 
+=head3 XCreateWindow
+
+Like L<X11::Xlib/XCreateWindow>, but returns a L<X11::Xlib::Window> object.
+
+=head3 XCreateSimpleWindow
+
+Like L<X11::Xlib::XCreateSimpleWindow>, but returns a L<X11::Xlib::Window> object.
+
+=cut
+
 sub XCreateWindow {
     $_[0]->get_cached_window( &X11::Xlib::XCreateWindow, autofree => 1);
 }
@@ -709,3 +722,23 @@ sub get_cached_window {
 }
 
 1;
+
+__END__
+
+=head1 AUTHOR
+
+Olivier Thauvin, E<lt>nanardon@nanardon.zarb.orgE<gt>
+
+Michael Conrad, E<lt>mike@nrdvana.netE<gt>
+
+=head1 COPYRIGHT AND LICENSE
+
+Copyright (C) 2009-2010 by Olivier Thauvin
+
+Copyright (C) 2017 by Michael Conrad
+
+This library is free software; you can redistribute it and/or modify
+it under the same terms as Perl itself, either Perl version 5.10.0 or,
+at your option, any later version of Perl 5 you may have available.
+
+=cut
