@@ -4,6 +4,10 @@ use warnings;
 use X11::Xlib::Display;
 require Scalar::Util;
 
+=head1 NAME
+
+X11::Xlib::Screen - Convenience wrapper around Display+ScreenID
+
 =head1 DESCRIPTION
 
 In ancient history, a C<Screen> represented one physical graphics device
@@ -136,3 +140,23 @@ sub match_visual_info {
 }
 
 1;
+
+__END__
+
+=head1 AUTHOR
+
+Olivier Thauvin, E<lt>nanardon@nanardon.zarb.orgE<gt>
+
+Michael Conrad, E<lt>mike@nrdvana.netE<gt>
+
+=head1 COPYRIGHT AND LICENSE
+
+Copyright (C) 2009-2010 by Olivier Thauvin
+
+Copyright (C) 2017 by Michael Conrad
+
+This library is free software; you can redistribute it and/or modify
+it under the same terms as Perl itself, either Perl version 5.10.0 or,
+at your option, any later version of Perl 5 you may have available.
+
+=cut
